/*****************************************************************************
 * davinci.h: decoder and encoder modules using the DaVinci DSP.
 *****************************************************************************
 * Copyright (C) 2008-2010 M2X BV
 * $Id: 56b30781fe9939e65d75414c90697eca4a188d7f $
 *
 * Authors: Antoine Cellerier <dionoea at videolan dot org>
 *          Jean-Paul Saman <jean-paul.saman at m2x dot nl>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
 *****************************************************************************/

#ifndef VLC_DAVINCI_H
#define VLC_DAVINCI_H

#define XDM_INCLUDE_DOT9_SUPPORT /* Support old 0.9 API */

#include <gnu/targets/std.h>
#include <xdc/std.h>
#include <ti/xdais/xdas.h>
#include <ti/xdais/dm/xdm.h>
#include <ti/sdo/ce/CERuntime.h>
#include <ti/sdo/ce/Engine.h>

#undef DEBUG_DAVINCI

#define ENC_CFG_PREFIX "sout-davinci-"
#define DEC_CFG_PREFIX "davinci-"

/* Wrapper for XDM version specific data */
typedef struct xdm_sys_t xdm_sys_t;

/* decoder */
int OpenVideoDecoder( vlc_object_t * );
void CloseVideoDecoder( vlc_object_t * );

int OpenAudioDecoder( vlc_object_t * );
void CloseAudioDecoder( vlc_object_t * );

/* encoder */
struct encoder_sys_t
{
    char *psz_ti_engine;
    const char *psz_codec;

    Engine_Handle engine;

    xdm_sys_t *xdm;

    block_t *(*pf_encode_video)( encoder_t *, picture_t * );
    block_t *(*pf_encode_audio)( encoder_t *, aout_buffer_t * );

    /* stats */
#ifdef DEBUG_FPS
    mtime_t i_pts_start;
    mtime_t i_pts_end;
    int     i_fps;

#endif
    int     i_pics;
};

int OpenEncoder( vlc_object_t * );
void CloseEncoder( vlc_object_t * );

int OpenEncoderAudio( encoder_t * );
void CloseEncoderAudio( encoder_t * );

int OpenEncoderAudio1( encoder_t * );
void CloseEncoderAudio1( encoder_t * );

int OpenEncoderVideo( encoder_t * );
void CloseEncoderVideo( encoder_t * );

//int OpenEncoderVideo1( encoder_t * );
//void CloseEncoderVideo1( encoder_t * );

/*****************************************************************************
 * Common stuff
 *****************************************************************************/
extern const char *ppsz_engine_error[];

#ifdef DEBUG_FPS
void calculate_fps( encoder_t *p_enc );
#endif

const char *davinci_GetExtendedError( XDAS_Int32 );
void davinci_PrintAvailableAlgorithms( vlc_object_t *, const char * );

/* Davinci XDM Versions */
enum davinci_xdm_version_e
{
        DAVINCI_XDM0_9 = 0x09,
        DAVINCI_XDM1_0 = 0x0A,
        DAVINCI_XDM1_1 = 0x0C,
};

bool GetDavinciDecoder( vlc_fourcc_t, int *, int *, const char **, const char ** );
bool GetDavinciEncoder( vlc_fourcc_t, int *, int *, const char **, const char ** );
bool GetVlcFourcc( const char *, int *, vlc_fourcc_t *, const char ** );
XDAS_Int32 VlcChromaToXdm( vlc_fourcc_t );

/* CE Runtime */
void davinci_RuntimeInit( vlc_object_t *p_this );
void davinci_RuntimeExit( vlc_object_t *p_this );

/* XDM 0.9 */
int davinci_AllocateBuffer( XDAS_Int32, XDAS_Int32 *, XDM_BufDesc * );
void davinci_FreeBuffer( XDM_BufDesc * );

/* XDM 1.0 */
int davinci_AllocateBuffer1( XDAS_Int32, XDAS_Int32 *, XDM1_BufDesc * );
void davinci_FreeBuffer1( XDM1_BufDesc * );

#endif

