/*****************************************************************************
 * fourcc.c: davinci <-> libvlc conversion routines
 *****************************************************************************
 * Copyright (C) 2009-2010 M2X BV
 * $Id: 88adee8ebdf176faa057deca5fb101d173afcb91 $
 *
 * Authors: Jean-Paul Saman <jpsaman@videolan.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
 *****************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include <vlc_common.h>
#include <vlc_codec.h>

#include "davinci.h"

/*****************************************************************************
 * Codec fourcc -> davinic_id mapping
 *****************************************************************************/

/* Enable codecs by changing #undef in #define here below
 * NOTE: Make sure it matches the codec pack you are using. */
#undef DAVINCI_HAS_MPEG1_CODEC

#define DAVINCI_HAS_MPEG2_CODEC
#define DAVINCI_HAS_MPEG4_CODEC
#define DAVINCI_HAS_H264_CODEC
#define DAVINCI_HAS_VC1_CODEC

#undef DAVINCI_HAS_G711_CODEC
#undef DAVINCI_HAS_MP3_CODEC
#define DAVINCI_HAS_AAC_CODEC

/* decoders names */
#define DAVINCI_DEC_MPEG1 "mpegdec"
#define DAVINCI_DEC_MPEG2 "mpeg2dec"
#define DAVINCI_DEC_MPEG4 "mpeg4dec"
#define DAVINCI_DEC_H264  "h264dec"
#define DAVINCI_DEC_VC1   "vc1dec"

#define DAVINCI_DEC_AAC    "aacdec"
#define DAVINIC_DEC_MP3    "mp3dec"
#define DAVINCI_DEC_G711   "g711dec"

/* encoders names */
#define DAVINCI_ENC_MPEG1 "mpegenc"
#define DAVINCI_ENC_MPEG2 "mpeg2enc"
#define DAVINCI_ENC_MPEG4 "mpeg4enc"
#define DAVINCI_ENC_H264  "h264enc"
#define DAVINCI_ENC_VC1   "vc1enc"

#define DAVINCI_ENC_AAC   "aacheenc"
#define DAVINCI_ENC_G711  "g711enc"

#define DAVINCI_NONE      ""

static const struct
{
    vlc_fourcc_t  i_fourcc;
    int  i_cat;
    const char psz_name[36];
    const char psz_decoder[10];
    const char psz_encoder[10];
    enum davinci_xdm_version_e i_xdm_version;
} codecs_table[] =
{
    /*
     * Video Codecs
     */
#ifdef DAVINCI_HAS_MPEG1_CODEC
    /* MPEG-1 Video */
    { VLC_FOURCC('m','p','1','v'),
      VIDEO_ES, "MPEG-1 Video",
      DAVINCI_DEC_MPEG1, DAVINCI_ENC_MPEG1, DAVINCI_XDM0_9 },
    { VLC_FOURCC('m','p','e','g'),
      VIDEO_ES, "MPEG-1 Video",
      DAVINCI_DEC_MPEG1, DAVINCI_ENC_MPEG1, DAVINCI_XDM0_9 },
    { VLC_FOURCC('m','p','g','1'),
      VIDEO_ES, "MPEG-1 Video",
      DAVINCI_DEC_MPEG1, DAVINCI_ENC_MPEG1, DAVINCI_XDM0_9 },
    { VLC_FOURCC('P','I','M','1'),
      VIDEO_ES, "Pinnacle DC1000 (MPEG-1 Video)",
      DAVINCI_DEC_MPEG1, DAVINCI_ENC_MPEG1, DAVINCI_XDM0_9 },
#endif
#ifdef DAVINCI_HAS_MPEG2_CODEC
    /* MPEG-2 Video */
    { VLC_FOURCC('m','p','2','v'),
      VIDEO_ES, "MPEG-2 Video",
      DAVINCI_DEC_MPEG2, DAVINCI_ENC_MPEG2, DAVINCI_XDM0_9 },
    { VLC_FOURCC('M','P','E','G'),
      VIDEO_ES, "MPEG-2 Video",
      DAVINCI_DEC_MPEG2, DAVINCI_ENC_MPEG2, DAVINCI_XDM0_9 },
    { VLC_FOURCC('m','p','g','v'),
      VIDEO_ES, "MPEG-2 Video",
      DAVINCI_DEC_MPEG2, DAVINCI_ENC_MPEG2, DAVINCI_XDM0_9 },
    { VLC_FOURCC('m','p','g','2'),
      VIDEO_ES, "MPEG-2 Video",
      DAVINCI_DEC_MPEG2, DAVINCI_ENC_MPEG2, DAVINCI_XDM0_9 },
    { VLC_FOURCC('h','d','v','1'),
      VIDEO_ES, "HDV 720p30 (MPEG-2 Video)",
      DAVINCI_DEC_MPEG2, DAVINCI_ENC_MPEG2, DAVINCI_XDM0_9 },
    { VLC_FOURCC('h','d','v','2'),
      VIDEO_ES, "Sony HDV (MPEG-2 Video)",
      DAVINCI_DEC_MPEG2, DAVINCI_ENC_MPEG2, DAVINCI_XDM0_9 },
    { VLC_FOURCC('h','d','v','3'),
      VIDEO_ES, "FCP HDV (MPEG-2 Video)",
      DAVINCI_DEC_MPEG2, DAVINCI_ENC_MPEG2, DAVINCI_XDM0_9 },
    { VLC_FOURCC('h','d','v','5'),
      VIDEO_ES, "HDV 720p25 (MPEG-2 Video)",
      DAVINCI_DEC_MPEG2, DAVINCI_ENC_MPEG2, DAVINCI_XDM0_9 },
    { VLC_FOURCC('h','d','v','6'),
      VIDEO_ES, "HDV 1080p24 (MPEG-2 Video)",
      DAVINCI_DEC_MPEG2, DAVINCI_ENC_MPEG2, DAVINCI_XDM0_9 },
    { VLC_FOURCC('h','d','v','7'),
      VIDEO_ES, "HDV 1080p25 (MPEG-2 Video)",
      DAVINCI_DEC_MPEG2, DAVINCI_ENC_MPEG2, DAVINCI_XDM0_9 },
    { VLC_FOURCC('h','d','v','8'),
      VIDEO_ES, "HDV 1080p30 (MPEG-2 Video)",
      DAVINCI_DEC_MPEG2, DAVINCI_ENC_MPEG2, DAVINCI_XDM0_9 },

    { VLC_FOURCC('m','x','5','n'),
      VIDEO_ES, "MPEG2 IMX NTSC 525/60 50mb/s (FCP)",
      DAVINCI_DEC_MPEG2, DAVINCI_ENC_MPEG2, DAVINCI_XDM0_9 },
    { VLC_FOURCC('m','x','5','p'),
      VIDEO_ES, "MPEG2 IMX PAL 625/60 50mb/s (FCP)",
      DAVINCI_DEC_MPEG2, DAVINCI_ENC_MPEG2, DAVINCI_XDM0_9 },
    { VLC_FOURCC('m','x','4','n'),
      VIDEO_ES, "MPEG2 IMX NTSC 525/60 40mb/s (FCP)",
      DAVINCI_DEC_MPEG2, DAVINCI_ENC_MPEG2, DAVINCI_XDM0_9 },
    { VLC_FOURCC('m','x','4','p'),
      VIDEO_ES, "MPEG2 IMX PAL 625/50 40mb/s (FCP)",
      DAVINCI_DEC_MPEG2, DAVINCI_ENC_MPEG2, DAVINCI_XDM0_9 },
    { VLC_FOURCC('m','x','3','n'),
      VIDEO_ES, "MPEG2 IMX NTSC 525/60 30mb/s (FCP)",
      DAVINCI_DEC_MPEG2, DAVINCI_ENC_MPEG2, DAVINCI_XDM0_9 },
    { VLC_FOURCC('m','x','3','p'),
      VIDEO_ES, "MPEG2 IMX NTSC 625/50 30mb/s (FCP)",
      DAVINCI_DEC_MPEG2, DAVINCI_ENC_MPEG2, DAVINCI_XDM0_9 },
    { VLC_FOURCC('x','d','v','2'),
      VIDEO_ES, "XDCAM HD 1080i60",
      DAVINCI_DEC_MPEG2, DAVINCI_ENC_MPEG2, DAVINCI_XDM0_9 },
    { VLC_FOURCC('A','V','m','p'),
      VIDEO_ES, "AVID IMX PAL",
      DAVINCI_DEC_MPEG2, DAVINCI_ENC_MPEG2, DAVINCI_XDM0_9 },
    /* ATI VCR2 */
    { VLC_FOURCC('V','C','R','2'),
      VIDEO_ES, "ATI VCR2 Video",
      DAVINCI_DEC_MPEG2, DAVINCI_ENC_MPEG2, DAVINCI_XDM0_9 },
    { VLC_FOURCC('M','M','E','S'),
      VIDEO_ES, "Matrox MPEG-2",
      DAVINCI_DEC_MPEG2, DAVINCI_ENC_MPEG2, DAVINCI_XDM0_9 },
    { VLC_FOURCC('m','m','e','s'),
      VIDEO_ES, "Matrox MPEG-2",
      DAVINCI_DEC_MPEG2, DAVINCI_ENC_MPEG2, DAVINCI_XDM0_9 },
    { VLC_FOURCC('P','I','M','2'),
      VIDEO_ES, "Pinnacle DC1000 (MPEG-2 Video)",
      DAVINCI_DEC_MPEG2, DAVINCI_ENC_MPEG2, DAVINCI_XDM0_9 },
#endif
#ifdef DAVINCI_HAS_MPEG4_CODEC
    /* MPEG-4 Video */
    { VLC_FOURCC('D','I','V','X'),
      VIDEO_ES, "MPEG-4 Video",
      DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4, DAVINCI_XDM0_9 },
    { VLC_FOURCC('d','i','v','x'),
      VIDEO_ES, "MPEG-4 Video",
      DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4, DAVINCI_XDM0_9 },
    { VLC_FOURCC('M','P','4','S'),
      VIDEO_ES, "MPEG-4 Video",
      DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4, DAVINCI_XDM0_9 },
    { VLC_FOURCC('m','p','4','s'),
      VIDEO_ES, "MPEG-4 Video",
      DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4, DAVINCI_XDM0_9 },
    { VLC_FOURCC('M','4','S','2'),
      VIDEO_ES, "MPEG-4 Video",
      DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4, DAVINCI_XDM0_9 },
    { VLC_FOURCC('m','4','s','2'),
      VIDEO_ES, "MPEG-4 Video",
      DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4, DAVINCI_XDM0_9 },
    /* XVID flavours */
    { VLC_FOURCC('x','v','i','d'),
      VIDEO_ES, "MPEG-4 Video",
      DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4, DAVINCI_XDM0_9 },
    { VLC_FOURCC('X','V','I','D'),
      VIDEO_ES, "MPEG-4 Video",
      DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4, DAVINCI_XDM0_9 },
    { VLC_FOURCC('X','v','i','D'),
      VIDEO_ES, "MPEG-4 Video",
      DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4, DAVINCI_XDM0_9 },
    { VLC_FOURCC('X','V','I','X'),
      VIDEO_ES, "MPEG-4 Video",
      DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4, DAVINCI_XDM0_9 },
    { VLC_FOURCC('x','v','i','x'),
      VIDEO_ES, "MPEG-4 Video",
      DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4, DAVINCI_XDM0_9 },
    /* DX50 */
    { VLC_FOURCC('D','X','5','0'),
      VIDEO_ES, "MPEG-4 Video",
      DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4, DAVINCI_XDM0_9 },
    { VLC_FOURCC('d','x','5','0'),
      VIDEO_ES, "MPEG-4 Video",
      DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4, DAVINCI_XDM0_9 },
    { VLC_FOURCC('B','L','Z','0'),
      VIDEO_ES, "Blizzard MPEG-4 Video",
      DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4, DAVINCI_XDM0_9 },
    { VLC_FOURCC('D','X','G','M'),
      VIDEO_ES, "Electronic Arts Game MPEG-4 Video",
      DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4, DAVINCI_XDM0_9 },
    { VLC_FOURCC('m','p','4','v'),
      VIDEO_ES, "MPEG-4 Video",
      DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4, DAVINCI_XDM0_9 },
    { VLC_FOURCC('M','P','4','V'),
      VIDEO_ES, "MPEG-4 Video",
      DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4, DAVINCI_XDM0_9 },
    { VLC_FOURCC( 4,  0,  0,  0 ),
      VIDEO_ES, "MPEG-4 Video",
      DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4, DAVINCI_XDM0_9 },
    { VLC_FOURCC('m','4','c','c'),
      VIDEO_ES, "MPEG-4 Video",
      DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4, DAVINCI_XDM0_9 },
    { VLC_FOURCC('M','4','C','C'),
      VIDEO_ES, "MPEG-4 Video",
      DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4, DAVINCI_XDM0_9 },
    { VLC_FOURCC('F','M','P','4'),
      VIDEO_ES, "MPEG-4 Video",
      DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4, DAVINCI_XDM0_9 },
    { VLC_FOURCC('f','m','p','4'),
      VIDEO_ES, "MPEG-4 Video",
      DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4, DAVINCI_XDM0_9 },
    /* 3ivx delta 3.5 Unsupported
     * putting it here gives extreme distorted images
    { VLC_FOURCC('3','I','V','1'),
      VIDEO_ES, "MPEG-4 Video",
      DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4, DAVINCI_XDM0_9 },
    { VLC_FOURCC('3','i','v','1'),
      VIDEO_ES, "MPEG-4 Video",
      DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4, DAVINCI_XDM0_9}, */
    /* 3ivx delta 4 */
    { VLC_FOURCC('3','I','V','2'),
      VIDEO_ES, "MPEG-4 Video",
      DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4, DAVINCI_XDM0_9 },
    { VLC_FOURCC('3','i','v','2'),
      VIDEO_ES, "MPEG-4 Video",
      DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4, DAVINCI_XDM0_9 },
    { VLC_FOURCC('U','M','P','4'),
      VIDEO_ES, "UB MPEG-4 Video",
      DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4, DAVINCI_XDM0_9 },
    { VLC_FOURCC('W','V','1','F'),
      VIDEO_ES, "MPEG-4 Video",
      DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4, DAVINCI_XDM0_9 },
    { VLC_FOURCC('S','E','D','G'),
      VIDEO_ES, "Samsung MPEG-4 Video",
      DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4, DAVINCI_XDM0_9 },
    { VLC_FOURCC('R','M','P','4'),
      VIDEO_ES, "REALmagic MPEG-4 Video",
      DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4, DAVINCI_XDM0_9 },
    { VLC_FOURCC('H','D','X','4'),
      VIDEO_ES, "Jomigo HDX4 (MPEG-4 Video)",
      DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4, DAVINCI_XDM0_9 },
    { VLC_FOURCC('h','d','x','4'),
      VIDEO_ES, "Jomigo HDX4 (MPEG-4 Video)",
      DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4, DAVINCI_XDM0_9 },
    { VLC_FOURCC('S','M','P','4'),
      VIDEO_ES, "Samsung SMP4 (MPEG-4 Video)",
      DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4, DAVINCI_XDM0_9 },
    { VLC_FOURCC('s','m','p','4'),
      VIDEO_ES, "Samsung SMP4 (MPEG-4 Video)",
      DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4, DAVINCI_XDM0_9 },
    { VLC_FOURCC('f','v','f','w'),
      VIDEO_ES, "FFmpeg MPEG-4",
      DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4, DAVINCI_XDM0_9 },
    { VLC_FOURCC('F','V','F','W'),
      VIDEO_ES, "FFmpeg MPEG-4",
      DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4, DAVINCI_XDM0_9 },
    { VLC_FOURCC('F','F','D','S'),
      VIDEO_ES, "FFDShow MPEG-4",
      DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4, DAVINCI_XDM0_9 },
    { VLC_FOURCC('V','I','D','M'),
      VIDEO_ES, "vidm 4.01 codec",
      DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4, DAVINCI_XDM0_9 },
    { VLC_FOURCC('D','C','O','D'),
      VIDEO_ES, "MPEG-4 Video",
      DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4, DAVINCI_XDM0_9 },
    { VLC_FOURCC('f','m','p','4'),
      VIDEO_ES, "MPEG-4 Video",
      DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4, DAVINCI_XDM0_9 },
    { VLC_FOURCC('M','V','X','M'),
      VIDEO_ES, "MPEG-4 Video",
      DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4, DAVINCI_XDM0_9 },
    { VLC_FOURCC('P','M','4','V'),
      VIDEO_ES, "MPEG-4 Video",
      DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4, DAVINCI_XDM0_9 },
    { VLC_FOURCC('f','m','p','4'),
      VIDEO_ES, "MPEG-4 Video",
      DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4, DAVINCI_XDM0_9 },
    { VLC_FOURCC('M','4','T','3'),
      VIDEO_ES, "MPEG-4 Video",
      DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4, DAVINCI_XDM0_9 },
    { VLC_FOURCC('G','E','O','X'),
      VIDEO_ES, "MPEG-4 Video",
      DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4, DAVINCI_XDM0_9 },
    { VLC_FOURCC('D','M','K','2'),
      VIDEO_ES, "MPEG-4 Video",
      DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4, DAVINCI_XDM0_9 },
    { VLC_FOURCC('D','I','G','I'),
      VIDEO_ES, "MPEG-4 Video",
      DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4, DAVINCI_XDM0_9 },
    { VLC_FOURCC('I','N','M','C'),
      VIDEO_ES, "MPEG-4 Video",
      DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4, DAVINCI_XDM0_9 },
    { VLC_FOURCC('S','N','4','0'),
      VIDEO_ES, "MPEG-4 Video",
      DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4, DAVINCI_XDM0_9 },
    { VLC_FOURCC('E','P','H','V'),
      VIDEO_ES, "MPEG-4 Video",
      DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4, DAVINCI_XDM0_9 },

    /* MSMPEG4 v1 */
    { VLC_FOURCC('D','I','V','1'),
      VIDEO_ES, "MS MPEG-4 Video v1",
      DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4, DAVINCI_XDM0_9 },
    { VLC_FOURCC('d','i','v','1'),
      VIDEO_ES, "MS MPEG-4 Video v1",
      DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4, DAVINCI_XDM0_9 },
    { VLC_FOURCC('M','P','G','4'),
      VIDEO_ES, "MS MPEG-4 Video v1",
      DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4, DAVINCI_XDM0_9 },
    { VLC_FOURCC('m','p','g','4'),
      VIDEO_ES, "MS MPEG-4 Video v1",
      DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4, DAVINCI_XDM0_9 },
    { VLC_FOURCC('m','p','4','1'),
      VIDEO_ES, "MS MPEG-4 Video v1",
      DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4, DAVINCI_XDM0_9 },

    /* MSMPEG4 v2 */
    { VLC_FOURCC('D','I','V','2'),
      VIDEO_ES, "MS MPEG-4 Video v2",
      DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4, DAVINCI_XDM0_9 },
    { VLC_FOURCC('d','i','v','2'),
      VIDEO_ES, "MS MPEG-4 Video v2",
      DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4, DAVINCI_XDM0_9 },
    { VLC_FOURCC('M','P','4','2'),
      VIDEO_ES, "MS MPEG-4 Video v2",
      DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4, DAVINCI_XDM0_9 },
    { VLC_FOURCC('m','p','4','2'),
      VIDEO_ES, "MS MPEG-4 Video v2",
      DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4, DAVINCI_XDM0_9 },

    /* MSMPEG4 v3 / M$ mpeg4 v3 */
    { VLC_FOURCC('M','P','G','3'),
      VIDEO_ES, "MS MPEG-4 Video v3",
      DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4, DAVINCI_XDM0_9 },
    { VLC_FOURCC('m','p','g','3'),
      VIDEO_ES, "MS MPEG-4 Video v3",
      DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4, DAVINCI_XDM0_9 },
    { VLC_FOURCC('d','i','v','3'),
      VIDEO_ES, "MS MPEG-4 Video v3",
      DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4, DAVINCI_XDM0_9 },
    { VLC_FOURCC('M','P','4','3'),
      VIDEO_ES, "MS MPEG-4 Video v3",
      DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4, DAVINCI_XDM0_9 },
    { VLC_FOURCC('m','p','4','3'),
      VIDEO_ES, "MS MPEG-4 Video v3",
      DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4, DAVINCI_XDM0_9 },
    /* DivX 3.20 */
    { VLC_FOURCC('D','I','V','3'),
      VIDEO_ES, "MS MPEG-4 Video v3",
      DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4, DAVINCI_XDM0_9 },
    { VLC_FOURCC('D','I','V','4'),
      VIDEO_ES, "MS MPEG-4 Video v3",
      DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4, DAVINCI_XDM0_9 },
    { VLC_FOURCC('d','i','v','4'),
      VIDEO_ES, "MS MPEG-4 Video v3",
      DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4, DAVINCI_XDM0_9 },
    { VLC_FOURCC('D','I','V','5'),
      VIDEO_ES, "MS MPEG-4 Video v3",
      DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4, DAVINCI_XDM0_9 },
    { VLC_FOURCC('d','i','v','5'),
      VIDEO_ES, "MS MPEG-4 Video v3",
      DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4, DAVINCI_XDM0_9 },
    { VLC_FOURCC('D','I','V','6'),
      VIDEO_ES, "MS MPEG-4 Video v3",
      DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4, DAVINCI_XDM0_9 },
    { VLC_FOURCC('d','i','v','6'),
      VIDEO_ES, "MS MPEG-4 Video v3",
      DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4, DAVINCI_XDM0_9 },
    /* Cool Codec */
    { VLC_FOURCC('C','O','L','1'),
      VIDEO_ES, "MS MPEG-4 Video v3",
      DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4, DAVINCI_XDM0_9 },
    { VLC_FOURCC('c','o','l','1'),
      VIDEO_ES, "MS MPEG-4 Video v3",
      DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4, DAVINCI_XDM0_9 },
    { VLC_FOURCC('C','O','L','0'),
      VIDEO_ES, "MS MPEG-4 Video v3",
      DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4, DAVINCI_XDM0_9 },
    { VLC_FOURCC('c','o','l','0'),
      VIDEO_ES, "MS MPEG-4 Video v3",
      DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4, DAVINCI_XDM0_9 },
    /* AngelPotion stuff */
    { VLC_FOURCC('A','P','4','1'),
      VIDEO_ES, "MS MPEG-4 Video v3",
      DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4, DAVINCI_XDM0_9 },
    /* 3ivx doctered divx files */
    { VLC_FOURCC('3','I','V','D'),
      VIDEO_ES, "MS MPEG-4 Video v3",
      DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4, DAVINCI_XDM0_9 },
    { VLC_FOURCC('3','i','v','d'),
      VIDEO_ES, "MS MPEG-4 Video v3",
      DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4, DAVINCI_XDM0_9 },
    /* who knows? */
    { VLC_FOURCC('3','V','I','D'),
      VIDEO_ES, "MS MPEG-4 Video v3",
      DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4, DAVINCI_XDM0_9 },
    { VLC_FOURCC('3','v','i','d'),
      VIDEO_ES, "MS MPEG-4 Video v3",
      DAVINCI_DEC_MPEG4, DAVINCI_ENC_MPEG4, DAVINCI_XDM0_9 },
#endif
#ifdef DAVINCI_HAS_H264_CODEC
    /* h264 */
    { VLC_FOURCC('h','2','6','4'),
      VIDEO_ES, "H264 - MPEG-4 AVC (part 10)",
      DAVINCI_DEC_H264, DAVINCI_ENC_H264, DAVINCI_XDM0_9 },
    { VLC_FOURCC('H','2','6','4'),
      VIDEO_ES, "H264 - MPEG-4 AVC (part 10)",
      DAVINCI_DEC_H264, DAVINCI_ENC_H264, DAVINCI_XDM0_9 },
    { VLC_FOURCC('x','2','6','4'),
      VIDEO_ES, "H264 - MPEG-4 AVC (part 10)",
      DAVINCI_DEC_H264, DAVINCI_ENC_H264, DAVINCI_XDM0_9 },
    { VLC_FOURCC('X','2','6','4'),
      VIDEO_ES, "H264 - MPEG-4 AVC (part 10)",
      DAVINCI_DEC_H264, DAVINCI_ENC_H264, DAVINCI_XDM0_9 },
    /* avc1: special case h264 */
    { VLC_FOURCC('a','v','c','1'),
      VIDEO_ES, "H264 - MPEG-4 AVC (part 10)",
      DAVINCI_DEC_H264, DAVINCI_ENC_H264, DAVINCI_XDM0_9 },
    { VLC_FOURCC('A','V','C','1'),
      VIDEO_ES, "H264 - MPEG-4 AVC (part 10)",
      DAVINCI_DEC_H264, DAVINCI_ENC_H264, DAVINCI_XDM0_9 },
    { VLC_FOURCC('V','S','S','H'),
      VIDEO_ES, "Vanguard VSS H264",
      DAVINCI_DEC_H264, DAVINCI_ENC_H264, DAVINCI_XDM0_9 },
    { VLC_FOURCC('V','S','S','W'),
      VIDEO_ES, "Vanguard VSS H264",
      DAVINCI_DEC_H264, DAVINCI_ENC_H264, DAVINCI_XDM0_9 },
    { VLC_FOURCC('v','s','s','h'),
      VIDEO_ES, "Vanguard VSS H264",
      DAVINCI_DEC_H264, DAVINCI_ENC_H264, DAVINCI_XDM0_9 },
    { VLC_FOURCC('D','A','V','C'),
      VIDEO_ES, "Dicas MPEGable H.264/MPEG-4 AVC",
      DAVINCI_DEC_H264, DAVINCI_ENC_H264, DAVINCI_XDM0_9 },
    { VLC_FOURCC('d','a','v','c'),
      VIDEO_ES, "Dicas MPEGable H.264/MPEG-4 AVC",
      DAVINCI_DEC_H264, DAVINCI_ENC_H264, DAVINCI_XDM0_9 },
#endif
#ifdef DAVINCI_HAS_VC1_CODEC
    /* vc-1 */
    { VLC_FOURCC('W','V','C','1'),
      VIDEO_ES, "Windows Media Video VC1",
      DAVINCI_DEC_VC1, DAVINCI_ENC_VC1, DAVINCI_XDM0_9 },
    { VLC_FOURCC('w','v','c','1'),
      VIDEO_ES, "Windows Media Video VC1",
      DAVINCI_DEC_VC1, DAVINCI_ENC_VC1, DAVINCI_XDM0_9 },
    { VLC_FOURCC('v','c','-','1'),
      VIDEO_ES, "Windows Media Video VC1",
      DAVINCI_DEC_VC1, DAVINCI_ENC_VC1, DAVINCI_XDM0_9 },
    { VLC_FOURCC('V','C','-','1'),
      VIDEO_ES, "Windows Media Video VC1",
      DAVINCI_DEC_VC1, DAVINCI_ENC_VC1, DAVINCI_XDM0_9 },
    { VLC_FOURCC('W','M','V','1'),
      VIDEO_ES, "Windows Media Video VC1",
      DAVINCI_DEC_VC1, DAVINCI_ENC_VC1, DAVINCI_XDM0_9 },
#endif

    /*
     * Audio Codecs
     */
#ifdef DAVINCI_HAS_VC1_CODEC
    /* WMVA is the VC-1 codec before the standardization proces,
       it is not bitstream compatible and deprecated  */
    { VLC_FOURCC('W','M','V','A'),
      AUDIO_ES, "Windows Media Video Advanced Profile",
      DAVINCI_DEC_VC1, DAVINCI_ENC_VC1, DAVINCI_XDM0_9 },
#endif
#ifdef DAVINCI_HAS_MP3_CODEC
    /* MPEG Audio layer 1/2/3 */
    { VLC_FOURCC('m','p','g','a'),
      AUDIO_ES, "MPEG Audio layer 1/2",
      DAVINIC_DEC_MP3, DAVINCI_NONE, DAVINCI_XDM0_9 },
    { VLC_FOURCC('m','p','3',' '),
      AUDIO_ES, "MPEG Audio layer 1/2/3",
      DAVINIC_DEC_MP3, DAVINCI_NONE, DAVINCI_XDM0_9 },
    { VLC_FOURCC('.','m','p','3'),
      AUDIO_ES, "MPEG Audio layer 1/2/3",
      DAVINIC_DEC_MP3, DAVINCI_NONE, DAVINCI_XDM0_9 },
    { VLC_FOURCC('M','P','3',' '),
      AUDIO_ES, "MPEG Audio layer 1/2/3",
      DAVINIC_DEC_MP3, DAVINCI_NONE, DAVINCI_XDM0_9 },
    { VLC_FOURCC('L','A','M','E'),
      AUDIO_ES, "MPEG Audio layer 1/2/3",
      DAVINIC_DEC_MP3, DAVINCI_NONE, DAVINCI_XDM0_9 },
#endif
#ifdef DAVINCI_HAS_AAC_CODEC
    /* AAC audio */
    { VLC_FOURCC('m','p','4','a'),
      AUDIO_ES, "MPEG AAC Audio",
      DAVINCI_DEC_AAC, DAVINCI_ENC_AAC, DAVINCI_XDM1_0 },
    { VLC_FOURCC('a','a','c',' '),
      AUDIO_ES, "MPEG AAC Audio",
      DAVINCI_DEC_AAC, DAVINCI_ENC_AAC, DAVINCI_XDM1_0 },
#endif
#ifdef DAVINCI_HAS_G711_CODEC
    /* PCM Audio */
    { VLC_FOURCC('s','8',' ',' '),
      AUDIO_ES, "PCM S8",
      DAVINCI_DEC_G711, DAVINCI_ENC_G711, DAVINCI_XDM0_9 },
    { VLC_FOURCC('u','8',' ',' '),
      AUDIO_ES, "PCM U8",
      DAVINCI_DEC_G711, DAVINCI_ENC_G711, DAVINCI_XDM0_9 },
    { VLC_FOURCC('s','1','6','l'),
      AUDIO_ES, "PCM S16 LE",
      DAVINCI_DEC_G711, DAVINCI_ENC_G711, DAVINCI_XDM0_9 },
    { VLC_FOURCC('s','1','6','b'),
      AUDIO_ES, "PCM S16 BE",
      DAVINCI_DEC_G711, DAVINCI_ENC_G711, DAVINCI_XDM0_9 },
    { VLC_FOURCC('u','1','6','l'),
      AUDIO_ES, "PCM U16 LE",
      DAVINCI_DEC_G711, DAVINCI_ENC_G711, DAVINCI_XDM0_9 },
    { VLC_FOURCC('u','1','6','b'),
      AUDIO_ES, "PCM U16 BE",
      DAVINCI_DEC_G711, DAVINCI_ENC_G711, DAVINCI_XDM0_9 },
    { VLC_FOURCC('s','2','4','l'),
      AUDIO_ES, "PCM S24 LE",
      DAVINCI_DEC_G711, DAVINCI_ENC_G711, DAVINCI_XDM0_9 },
    { VLC_FOURCC('s','2','4','b'),
      AUDIO_ES, "PCM S24 BE",
      DAVINCI_DEC_G711, DAVINCI_ENC_G711, DAVINCI_XDM0_9 },
    { VLC_FOURCC('u','2','4','l'),
      AUDIO_ES, "PCM U24 LE",
      DAVINCI_DEC_G711, DAVINCI_ENC_G711, DAVINCI_XDM0_9 },
    { VLC_FOURCC('u','2','4','b'),
      AUDIO_ES, "PCM U24 BE",
      DAVINCI_DEC_G711, DAVINCI_ENC_G711, DAVINCI_XDM0_9 },
    { VLC_FOURCC('s','3','2','l'),
      AUDIO_ES, "PCM S32 LE",
      DAVINCI_DEC_G711, DAVINCI_ENC_G711, DAVINCI_XDM0_9 },
    { VLC_FOURCC('s','3','2','b'),
      AUDIO_ES, "PCM S32 BE",
      DAVINCI_DEC_G711, DAVINCI_ENC_G711, DAVINCI_XDM0_9 },
    { VLC_FOURCC('u','3','2','l'),
      AUDIO_ES, "PCM U32 LE",
      DAVINCI_DEC_G711, DAVINCI_ENC_G711, DAVINCI_XDM0_9 },
    { VLC_FOURCC('u','3','2','b'),
      AUDIO_ES, "PCM U32 BE",
      DAVINCI_DEC_G711, DAVINCI_ENC_G711, DAVINCI_XDM0_9 },
    { VLC_FOURCC('a','l','a','w'),
      AUDIO_ES, "PCM ALAW",
      DAVINCI_DEC_G711, DAVINCI_ENC_G711, DAVINCI_XDM0_9 },
    { VLC_FOURCC('u','l','a','w'),
      AUDIO_ES, "PCM ULAW",
      DAVINCI_DEC_G711, DAVINCI_ENC_G711, DAVINCI_XDM0_9 },
    { VLC_FOURCC('d','a','u','d'),
      AUDIO_ES, "PCM ULAW",
      DAVINCI_DEC_G711, DAVINCI_ENC_G711, DAVINCI_XDM0_9 },
#endif
    { 0, 0, "", "", "", 0 }
};

bool GetDavinciDecoder( vlc_fourcc_t i_fourcc, int *pi_cat, int *pi_xdm,
                    const char **ppsz_codec, const char **ppsz_name )
{
    for( unsigned i = 0; codecs_table[i].i_fourcc != 0; i++ )
    {
        if( codecs_table[i].i_fourcc == i_fourcc )
        {
            if( pi_cat ) *pi_cat = codecs_table[i].i_cat;
            if( pi_xdm ) *pi_xdm = codecs_table[i].i_xdm_version;
            if( ppsz_codec ) *ppsz_codec = (char *)codecs_table[i].psz_decoder;
            if( ppsz_name ) *ppsz_name = (char *)codecs_table[i].psz_name;

            return true;
        }
    }
    return false;
}

bool GetDavinciEncoder( vlc_fourcc_t i_fourcc, int *pi_cat, int *pi_xdm,
                    const char **ppsz_codec, const char **ppsz_name )
{
    for( unsigned i = 0; codecs_table[i].i_fourcc != 0; i++ )
    {
        if( codecs_table[i].i_fourcc == i_fourcc )
        {
            if( pi_cat ) *pi_cat = codecs_table[i].i_cat;
            if( pi_xdm ) *pi_xdm = codecs_table[i].i_xdm_version;
            if( ppsz_codec ) *ppsz_codec = (char *)codecs_table[i].psz_encoder;
            if( ppsz_name ) *ppsz_name = (char *)codecs_table[i].psz_name;

            return true;
        }
    }
    return false;
}

bool GetVlcFourcc( const char *ppsz_codec, int *pi_cat,
                   vlc_fourcc_t *pi_fourcc, const char **ppsz_name )
{
    for( unsigned i = 0; codecs_table[i].i_fourcc != 0; i++ )
    {
        if( strncmp( codecs_table[i].psz_decoder, ppsz_codec, 10 ) == 0 )
        {
            if( pi_cat ) *pi_cat = codecs_table[i].i_cat;
            if( pi_fourcc ) *pi_fourcc = codecs_table[i].i_fourcc;
            if( ppsz_name ) *ppsz_name = codecs_table[i].psz_name;

            return true;
        }
        else if( strncmp( codecs_table[i].psz_encoder, ppsz_codec, 10 ) == 0 )
        {
            if( pi_cat ) *pi_cat = codecs_table[i].i_cat;
            if( pi_fourcc ) *pi_fourcc = codecs_table[i].i_fourcc;
            if( ppsz_name ) *ppsz_name = codecs_table[i].psz_name;

            return true;
        }
    }
    return false;
}

XDAS_Int32 VlcChromaToXdm( vlc_fourcc_t i_chroma )
{
    switch( i_chroma )
    {
        case VLC_FOURCC('I','4','2','0'):
            return XDM_YUV_420P;

        case VLC_FOURCC('I','4','1','1'):
            return XDM_YUV_411P;

        case VLC_FOURCC('I','4','2','2'):
            return XDM_YUV_422P;

        case VLC_FOURCC('I','4','4','4'):
            return XDM_YUV_444P;

        case VLC_FOURCC('Y','U','Y','V'):
            return XDM_YUV_422IBE;

        case VLC_FOURCC('U','Y','V','Y'):
            return XDM_YUV_422ILE;

        case VLC_FOURCC('G','R','E','Y'):
            return XDM_GRAY;

#if 0
        /* FIXME: not sure about this one */
        case VLC_FOURCC('R','V','2','4'):
        case VLC_FOURCC('R','V','3','2'):
            return XDM_RGB;
#endif

        default:
            return XDM_CHROMA_NA;
    }
}

